#KEGGserver <- SOAPServer("http://soap.genome.ad.jp/keggapi/request.cgi")
#KEGGxmlns = "SOAP/KEGG"
#KEGGaction = "SOAP/KEGG"

get.best.neighbors.by.gene <- function(genes.id, start, max.results){
    return(getBestNeighbors(genes.id, start, max.results, "best"))
}

get.best.best.neighbors.by.gene <- function(genes.id, start, max.results){
    return(getBestNeighbors(genes.id, start, max.results, "best_best"))
}

getBestNeighbors <- function(genes.id, start, max.results,
                             what = c("best", "best_best")){
    what <- match.arg(what)

    if(what == "best"){
        neighbors <-  .SOAP(KEGGserver, "get_best_neighbors_by_gene",
                            .soapArgs=list('genes_id' = genes.id,
                            start = start,  'max_results' = max.results),
                            action=KEGGaction, xmlns = KEGGxmlns)
    }else{
        neighbors <-  .SOAP(KEGGserver, "get_best_best_neighbors_by_gene",
                            .soapArgs=list('genes_id' = genes.id,
                            start = start,
                                 'max_results' = max.results),
                            action=KEGGaction, xmlns = KEGGxmlns)
    }

    return(neighbors)
}

get.paralogs.by.gene <- function(genes.id, start, max.results){
    return(.SOAP(KEGGserver, "get_paralogs_by_gene",
                 .soapArgs=list('genes_id' = genes.id, start = start,
                      'max_results' = max.results),
                 action=KEGGaction, xmlns = KEGGxmlns))
}

get.genes.by.pathway <- function(pathway.id) {
    return(unlist(.SOAP(KEGGserver, "get_genes_by_pathway",
                 .soapArgs=list('pathway_id' = pathway.id), action=KEGGaction,
                        xmlns = KEGGxmlns)))
}

get.enzymes.by.pathway <- function(pathway.id){
    return(unlist(.SOAP(KEGGserver, "get_enzymes_by_pathway",
                 .soapArgs=list('pathway_id' = pathway.id), action=KEGGaction,
                        xmlns = KEGGxmlns)))
}

get.compounds.by.pathway <- function(pathway.id){
    return(unlist(.SOAP(KEGGserver, "get_compounds_by_pathway",
                       .soapArgs=list('pathway_id' = pathway.id), action=KEGGaction,
                        xmlns = KEGGxmlns)))
}

get.reactions.by.pathway <- function(pathway.id){
    return(unlist(.SOAP(KEGGserver, "get_reactions_by_pathway",
                        .soapArgs=list('pathway_id' = pathway.id), action=KEGGaction,
                        xmlns = KEGGxmlns)))
}

get.motifs.by.gene <- function(genes.id, db){
   .SOAP(KEGGserver, "get_motifs_by_gene", .soapArgs=list('gene_id' = genes.id,
         db = db), action=KEGGaction, xmlns = KEGGxmlns)
}

get.genes.by.motifs <- function(motif.id.list, start, max.results){
    genes <- matrix(unlist(.SOAP(KEGGserver, "get_genes_by_motifs",
                    .soapArgs=list('motif_id_list' = motif.id.list,
                    start = start, 'max_results' = max.results),
                    action=KEGGaction, xmlns = KEGGxmlns),
                    use.names = FALSE), ncol = 2, byrow = TRUE)

    temp <- genes[,2]
    names(temp) <- genes[,1]

    return(temp)
}

list.databases <- function(){
  db <- matrix(unlist(.SOAP(KEGGserver, "list_databases", "",
             action = KEGGaction, xmlns = KEGGxmlns), use.names = FALSE))

}

list.organisms <- function(){
    orgs <- matrix(unlist(.SOAP(KEGGserver, "list_organisms", "",
                 action = KEGGaction, xmlns = KEGGxmlns),
                 use.names = FALSE), ncol = 2, byrow = TRUE)
    temp <- orgs[,2]
    names(temp) <- orgs[,1]

    return(temp)
}

list.pathways <- function(org){
    pathways <- matrix(unlist(.SOAP(KEGGserver, "list_pathways",
                       .soapArgs=list(org = org), action = KEGGaction,
                       xmlns = KEGGxmlns), use.name = FALSE),
                       ncol = 2, byrow = TRUE)
    temp <- pathways[,2]
    names(temp) <- gsub("- +.*$", "", pathways[,1])

    return(temp)
}

get.genes.by.organism <- function(org, start, max.results){
    return(unlist(.SOAP(KEGGserver, "get_genes_by_organism",
                        .soapArgs=list(org = org,
          start = start, 'max_results' = max.results),
          action = KEGGaction, xmlns = KEGGxmlns)))
}

mark.pathway.by.objects <- function(pathway.id, object.id.list){
    return(.SOAP(KEGGserver, "mark_pathway_by_objects",
                        .soapArgs=list('pathway_id' = pathway.id,
                             'object_id_list' = object.id.list),
                        action = KEGGaction, xmlns = KEGGxmlns))
}

color.pathway.by.objects <- function(pathway.id, object.id.list,
                                     fg.color.list, bg.color.list){
     return(.SOAP(KEGGserver, "color_pathway_by_objects",
                        .soapArgs=list('pathwya_id' = pathway.id,
                             'object_id_list' = object.id.list,
                             'fg_color_list' = fg.color.list,
                             'bg_color_list' = bg.color.list),
                        action = KEGGaction, xmlns = KEGGxmlns))
}

get.pathways.by.genes <- function(genes.id.list){
    return(unlist(.SOAP(KEGGserver, "get_pathways_by_genes",
                 .soapArgs=list('genes_id_list' = genes.id.list),
                 action = KEGGaction, xmlns = KEGGxmlns)))
}

get.pathways.by.enzymes <- function(enzyme.id.list){
    return(unlist(.SOAP(KEGGserver, "get_pathways_by_enzymes",
                 .soapArgs=list('enzymes_id_list' = enzyme.id.list),
                 action = KEGGaction, xmlns = KEGGxmlns)))
}

get.pathways.by.compounds <- function(compound.id.list){
    return(unlist(.SOAP(KEGGserver, "get_pathways_by_compounds",
                 .soapArgs=list('compounds_id_list' = compound.id.list),
                 action = KEGGaction, xmlns = KEGGxmlns)))
}

get.pathways.by.reactions <- function(reaction.id.list){
    return(unlist(.SOAP(KEGGserver, "get_pathways_by_reactions",
                 .soapArgs=list('reactions_id_list' = reaction.id.list),
                 action = KEGGaction, xmlns = KEGGxmlns)))
}
